==> pydev_toy-0.0.11-py3-none-any.whl: pydev/utils.py <==
"""pydev utils"""

import os
import sys
import json
import tomli
import tomli_w
import semver
import jmespath
import subprocess

from pathlib import Path

from functools import lru_cache

from packaging.version import Version

from urllib import request
from urllib.error import HTTPError


def get_python():
    """
    Python executable path for sub commands

    Return:
        Python executable where the package is installed.
        This could be changed to use the python from
        the system path or a virtual env.
    """
    return sys.executable


@lru_cache
def get_project_root(strict=False):
    """Walk up to find pyproject.toml"""

    cwd = Path.cwd()

    for path in cwd, *cwd.parents:
        if path.joinpath("pyproject.toml").exists():
            return path

    if strict:
        raise FileNotFoundError("pyproject.toml")


def run_command(command, echo=True, strict=False, chgdir=True):
    """Run shell command"""

    if echo:
        print(command)

    cwd = None
    if chgdir:
        cwd = get_project_root()
        if cwd is None:
            print("pyproject.toml file not found!")
            exit(1)

    rc = subprocess.run(command, cwd=cwd, shell=True)

    if strict and rc != 0:
        raise RuntimeError("Command failed!")


@lru_cache
def load_config():
    """Load pyproject.toml file"""

    pyproject = Path("pyproject.toml").resolve(strict=True)

    with pyproject.open("rb") as f:
        return tomli.load(f)


def get_config(item: str):
    """Query pyproject.toml file"""

    data = load_config()

    for i in item.split("."):
        data = data.get(i, None)
        if data is None:
            break

    return data


def search_path(pattern: str, path=None):
    """Search items in path"""

    if path is None:
        path = os.getenv("PATH")
    if isinstance(path, str):
        path = path.split(os.pathsep)
    if isinstance(path, os.PathLike):
        path = [path]

    for p in path:
        p = Path(p)
        yield from p.glob(pattern)


def system_python(version=None):
    """Python in the system path"""
    if not version:
        version = "3"

    path = os.getenv("PATH", "/usr/bin:/bin:/usr/sbin:/sbin:/usr/local/bin")
    path = path.split(os.pathsep)

    pattern = "python" + version
    items = [i for p in path for i in Path(p).glob(pattern)]

    return next(items, None)


def pyenv_versions():
    """pyenv versions"""

    pyenv_root = os.getenv("PYENV_ROOT", "~/.pyenv")
    pyenv_root = Path(pyenv_root).expanduser()

    return pyenv_root.glob("versions/*")


def pyenv_python(version: str = None) -> Path:
    """pyenv binary for version prefix"""

    if not version:
        version = "3.*"
    elif version.count(".") < 2:
        version += ".*"

    pyenv_root = os.getenv("PYENV_ROOT", "~/.pyenv")
    pyenv_root = Path(pyenv_root).expanduser()

    if not pyenv_root.exists():
        return None

    pattern = f"versions/{version}/bin/python"

    return next(pyenv_root.glob(pattern), None)


def conda_envs():
    """conda environments"""

    conda_root = os.getenv("CONDA_PREFIX", "~/miniconda3")
    conda_root = Path(conda_root).expanduser()

    return [p for p in conda_root.glob("envs/*") if p.joinpath("bin/python").exists()]


def conda_python(name: str = None) -> Path:
    """conda binary for named env"""

    conda_root = os.getenv("CONDA_PREFIX", "~/miniconda3")
    conda_root = Path(conda_root).expanduser()

    if name:
        conda_env = conda_root.joinpath(f"envs/{name}")
    else:
        conda_env = conda_root

    if conda_env.exists():
        return conda_env.joinpath("bin/python")


def which_python(version=None, target=None):
    """Locate python matching version/target"""
    if target == "pyenv":
        return pyenv_python(version)

    if target == "conda":
        return conda_python(version)

    if target in ("system", None):
        return system_python(version)


def confirm_choice(message, default: bool = None):
    prompt = f"{message} (yes/no):"

    while True:
        user_input = input(prompt)
        if user_input.lower() in ("y", "yes"):
            return True
        if user_input.lower() in ("n", "no"):
            return False
        if user_input == "" and default is not None:
            return default



def pypi_releases(name):
    """List of version on pypi"""
    url = f"https://pypi.org/pypi/{name}/json"
    try:
        res = request.urlopen(url)
        data = json.load(res)
        releases = data.get("releases", [])
        return sorted(releases, key=Version, reverse=True)
    except HTTPError:
        return []


def check_pypi_version(name, version):
    """Check if version is on pypi"""
    releases = pypi_releases(name)
    return version in releases

def bump_version():
    """Bump static version in pyproject.toml"""
    project_root = get_project_root()
    pyproject = project_root.joinpath("pyproject.toml").resolve(strict=True)
    data = tomli.loads(pyproject.read_text())
    version = jmespath.search("project.version", data)
    if not version:
        raise ValueError("Could not find project.version setting")
    version = semver.VersionInfo.parse(version)
    new_version = version.bump_patch()
    data['project']['version'] = str(new_version)
    print(f"Updating version to {new_version} ...")
    output = tomli_w.dumps(data)
    pyproject.write_text(output)


def build_project():
    """Build project wheel"""
    python = get_python()
    project_root = get_project_root()
    if project_root.joinpath("setup.py").exists():
        target = "sdist"
    else:
        target = "wheel"

    run_command(f"{python} -m build --{target}")

==> pydev_toy-0.0.12-py3-none-any.whl: pydev/utils.py <==
"""pydev utils"""

import os
import re
import sys
import json
import tomli
import shutil
import semver
import subprocess

from pathlib import Path

from functools import lru_cache

from packaging.version import Version

from urllib import request
from urllib.error import HTTPError


@lru_cache
def get_project_root(strict=False):
    """Walk up to find pyproject.toml"""

    cwd = Path.cwd()

    for path in cwd, *cwd.parents:
        if path.joinpath("pyproject.toml").exists():
            return path

    if strict:
        raise FileNotFoundError("pyproject.toml")


def run_command(command, echo=True, strict=False, chgdir=True):
    """Run shell command"""

    if echo:
        print(command)

    cwd = None
    if chgdir:
        cwd = get_project_root()
        if cwd is None:
            print("pyproject.toml file not found!")
            exit(1)

    rc = subprocess.run(command, cwd=cwd, shell=True)

    if strict and rc != 0:
        raise RuntimeError("Command failed!")


@lru_cache
def load_config():
    """Load pyproject.toml file"""

    pyproject = Path("pyproject.toml").resolve(strict=True)

    with pyproject.open("rb") as f:
        return tomli.load(f)


def get_config(item: str):
    """Query pyproject.toml file"""

    data = load_config()

    for i in item.split("."):
        data = data.get(i, None)
        if data is None:
            break

    return data


def search_path(pattern: str, path=None):
    """Search items in path"""

    if path is None:
        path = os.getenv("PATH")
    if isinstance(path, str):
        path = path.split(os.pathsep)
    if isinstance(path, os.PathLike):
        path = [path]

    for p in path:
        p = Path(p)
        yield from p.glob(pattern)


def confirm_choice(message, default: bool = None):
    prompt = f"{message} (yes/no):"

    while True:
        user_input = input(prompt)
        if user_input.lower() in ("y", "yes"):
            return True
        if user_input.lower() in ("n", "no"):
            return False
        if user_input == "" and default is not None:
            return default


def pypi_releases(name):
    """List of version on pypi"""
    url = f"https://pypi.org/pypi/{name}/json"
    try:
        res = request.urlopen(url)
        data = json.load(res)
        releases = data.get("releases", [])
        return sorted(releases, key=Version, reverse=True)
    except HTTPError:
        return []


def bump_version():
    """Bump patch version in pyproject"""
    project_root = get_project_root()
    pyproject = project_root.joinpath("pyproject.toml").resolve(strict=True)
    buffer = pyproject.read_text()
    pattern = re.compile(
        r'^version \s* = \s* "(.+)" \s*', flags=re.VERBOSE | re.MULTILINE
    )
    if match := pattern.search(buffer):
        version = semver.VersionInfo.parse(match.group(1))
    else:
        raise ValueError("Could not find version setting")
    new_version = version.bump_patch()
    print(f"Updating version to {new_version} ...")
    output = pattern.sub(f'version = "{new_version}"\n', buffer)
    pyproject.write_text(output)


def already_released():
    name = get_config("project.name")
    version = get_config("project.version")
    releases = pypi_releases(name)
    return version in releases


def build_project(*, clean=False, auto_bump=False):
    """Build project wheel"""

    # bump version if needed
    if auto_bump and already_released():
        bump_version()

    python = sys.executable
    project_root = get_project_root(strict=True)
    dist = project_root.joinpath("dist")

    # clean dist folder if present
    if clean and dist.is_dir():
        print(f"rmtree {dist}")
        shutil.rmtree(dist)

    # pick target depending on setup config
    if project_root.joinpath("setup.py").exists():
        target = "sdist"
    else:
        target = "wheel"

    run_command(f"{python} -m build --{target}")
